#ifndef UART_H
#define UART_H
/*-------------------------------------------------------------*/
/*				Includes and dependencies						*/
/*-------------------------------------------------------------*/
#include "UARTPort.h"
/*-------------------------------------------------------------*/
/*				Macros and definitions							*/
/*-------------------------------------------------------------*/

/*-------------------------------------------------------------*/
/*				Typedefs enums & structs						*/
/*-------------------------------------------------------------*/
/**
Defines the values for the transmitted data bits
*/
enum enUARTDatabits
{
	E_UART_DATABITS_8,
	E_UART_DATABITS_9,
};

/**
Defines the parity checking modes
*/
enum enUARTParity
{
	E_UART_PARITY_NONE,
	E_UART_PARITY_EVEN,
	E_UART_PARITY_ODD,
};

/**
Number of stop bits to send/receive
*/
enum enUARTStopBits
{
	E_UART_STOPBITS_1,
	E_UART_STOPBITS_2,
};

/**
Defines the modules available for use with this library. Not all modules are
implemented on some devices. If a module is not implemented the xUARTInitialize()
call will return NULL.
*/
enum enUARTModules
{
    E_UART_NULL,
	E_UART_1 = 1,
	E_UART_2 = 2,
	E_UATR_3 = 3,
	E_UART_4 = 4,
	E_UART_5 = 5,
};
/*-------------------------------------------------------------*/
/*				Function prototypes								*/
/*-------------------------------------------------------------*/
/**
@brief Prepares the UART hardware and I/O ports for operation.

Prepares the hardware for use, initializes the IO ports (if necesary)
prepares the required data structures and Operating System resources (if needed)
and initializes the needed registers. This function returns a handle to the UART
resource that will be used to get access to other API calls.

If the specified module is not available, or the initialization progress fails
by any other cause this function will return NULL.

@param enUARTModules The UART module to initialize.

@return A resource handle that identifies the UART module, or NULL if the device
doesn´t exist or it cannot be initialized.
*/
xUARTHandle xUARTInitialize( enum enUARTModules );

/**
@brief Sets the configuration parameters for the UART module.

This function sets the configuration for the UART operation. If the UART cannot
be configured using the provided parameters, due to incorrect or unsupported
settings, this function will return FALSE.

The UART module should be closed before calling this function.

@param xUART The UART hardware descriptor of the module to be configured
@param eDataBits The data bits to be transmitted or received
@param eParity The parity checking method
@param eStopBits The number of stop bits

@return Returns TRUE if the UART module was succesfully configured, returns
false if the configuration is invalid or the underlying harware cannot accept
the given parameters.
*/
BOOL xUARTConfigure( xUARTHandle xUART, enum enUARTDatabits eDataBits, enum enUARTParity eParity, enum enUARTStopBits eStopBits );

/**
@brief Sets the operating baudrate for the UART

Configures the communication speed of the given UART module.

The UART should be closed before calling this function.

@param xUART The UART hardware descriptor of the module to be configured
@param ulBaudRate The desired baudrate for the UART communications

@return Returns TRUE if the Baudrate is succesfully configured, returns FALSE if
the baudrate is out of range or if the baudrate configuration fails.
*/
BOOL xUARTSetBaudRate( xUARTHandle xUART, unsigned long ulBaudRate );

/**
@brief Enables UART module operation

This function enables the UART module operation, the module takes control over
the UART pins. This function returns TRUE if the UART module is not bussy.

This function can be used as a semaphore when multiple programs or tasks use one
UART module in order to implement mutual exclusion between them.

@param xUART The UART hardware descriptor of the module to open

@return Returns TRUE if the UART is available to use and ready to use. Returns
false if the module is bussy (already open).
*/
BOOL xUARTOpen( xUARTHandle xUART );

/**
@brief Disables UART module operation

Closes the UART module and returns the semaphore for mutually exclusive access
to the UART resource.

@param xUART The UART hardware descriptor of the module to close

@return Returns TRUE if the UART module was closed succesfully, false otherwise
*/
BOOL xUARTClose( xUARTHandle xUART );

/**
@brief Writes a single byte to the UART.

Writes a data bit to the UART. This is a blocking call, program execution will
stop untill the data is transmitted.

@param xUART The UART hardware descriptor of the module to write
@param xTxChar The data byte to send
*/
void vUARTWrite( xUARTHandle xUART, BYTE xTxChar );

/**
@brief Writes a single data word to the UART.

Writes a single data word to the UART. This function is intended to be used when
transmitting 9 bit data. This is a blocking call, program execution will stop
untill the data is transmitted.

@param xUART The UART hardware descriptor of the module to write
@param xTxChar The data byte to send
*/
void vUARTWriteWord( xUARTHandle xUART, WORD xTxChar );

/**
@brief Reads a single byte from the UART.

Reads a single data byte from the UART. This is a blocking call, program
execution will stop untill a data byte is received.

@param xUART The UART hardware descriptor of the module to read

@return Returns the received data byte
*/	
BYTE xUARTRead( xUARTHandle xUART );

/**
@brief Reads a single word from the UART.

Reads a single data word from the UART. This function is intended to be used 
when receiving 9 bit data. This is a blocking call, program execution will stop
untill a data word is received.

@param xUART The UART hardware descriptor of the module to read

@return Returns the received data word
*/
WORD xUARTReadWord( xUARTHandle xUART );

/**
@brief Writes an array of bytes to the UART

This function writes an array of bytes to the UART. This is a blocking call,
program execution will stop untill data is transmitted.

@param xUART The UART hardware descriptor of the module to write
@param pcTxBuffer The buffer containing the data to send
@param usBufferLength The lenght of the data to send
*/
void vUARTWriteArray( xUARTHandle xUART, const char * pcTxBuffer, unsigned short usBufferLength );

/**
@brief Reads an array of bytes from the UART

This function reads an array of bytes from the UART. This is a blocking call,
program execution will stop untill data is received.

@param xUART The UART hardware descriptor of the module to read
@param pcRxBuffer The buffer to hold the received data
@param usBufferLength The number of bytes to receive
*/
void vUARTReadArray( xUARTHandle xUART, char * pcRxBuffer, unsigned short usBufferLength );

/**
@brief Writes a null terminated string to the UART.

This function writes a string to the UART, the string should be null terminated.
This is a blocking call, program execution will stop untill data is transmitted.

@param xUART The UART hardware descriptor of the module to write
@param pcString Pointer to the string to send
*/
void vUARTPuts( xUARTHandle xUART, const char * pcString );

/**
@brief Reads a null terminated string from the UART.

This function reads a string from the UART, the string should be null terminated.
This is a blocking call, program execution will stop untill data is received.

@param The UART hardware descriptor of the module to write
@param pcBufer Pointer to the buffer to store the received string
@param usBufferLength The lenght of the receive buffer, max lenght of the string
*/
void vUARTGets( xUARTHandle xUART, char * pcBuffer, unsigned short );

/**
@brief Checks if data is available to be read from UART.

Checks if data is available on the UART receive buffer to be read inmediately.
This can be used to check for incomming data without blocking program execution.
If this function returns TRUE, it means that at least 1 data byte can be read
using xUARTRead().

@param The UART hardware descriptor of the module to check

@return Returns TRUE if data is available to receive, false otherwise.
*/
BOOL xUARTAvailable( xUARTHandle xUART );

#endif
// End of Header file
