#include "UART.h"

/*
Implementation of serial communications for PIC devices with AUSART module PIC16F family
*/

xUARTHandle xUARTInitialize( enum enUARTModules eUART )
{
	// Initialize IO ports
	// On PIC16F devices USART I/O Ports should be configured as Inputs
	// On PIC16F family all devices have only one hardware USART
	if( eUART != 1 )
		return 0;
	// Reset UART registers
	TXSTA = 0x00;
	RCSTA = 0x00;
	SPBRG = 0x00;
	
	// Set default baudrate
	if( !xUARTSetBaudRate( eUART, 9600 ) )
		return 0;
	// Apply default UART configuration
	if( !xUARTConfigure( eUART, E_UART_DATABITS_8, E_UART_PARITY_NONE, E_UART_STOPBITS_1 ) )
		return 0;
	// Return module identifier
	return eUART;
}

BOOL xUARTConfigure(xUARTHandle xUART, enum enUARTDatabits eDataBits, enum enUARTParity eParity, enum enUARTStopBits eStopBits)
{
	if( xUART == 1 )
	{
		// The  USART on PIC16F devices only supports 1 stopbit and has no parity
		// generator built in, parity however can be emulated using the ninth bit
		if( eParity != E_UART_PARITY_NONE || eStopBits != E_UART_STOPBITS_1 )
			return FALSE;

		TXSTAbits.BRGH = 1;
		TXSTAbits.SYNC = 0;
		//RCSTAbits.SPEN = 1;
		
		// Check if 9 bit data is enabled
		if ( eDataBits == E_UART_DATABITS_9 )
		{
			TXSTA |= (1<<6);
			RCSTA |= (1<<6);
		}
		
		//TXSTAbits.TXEN = 1;
		//RCSTAbits.CREN = 1;
		
		return TRUE;
	}
	return FALSE;
}

#define TCY_CLOCK   2000000

BOOL xUARTSetBaudRate(xUARTHandle xUART, unsigned long ulBaudRate)
{
	if( xUART == 1 )
	{
		SPBRG =  ( TCY_CLOCK / (4 * ulBaudRate ) ) - 1;
		return TRUE;
	}
	return FALSE;
}

BOOL xUARTOpen( xUARTHandle xUART )
{
	if( xUART == 1 )
	{
		//RCSTA |= UART_SPEN_BIT; 	// Set SPEN bit
		RCSTAbits.SPEN = 1;
		TXSTAbits.TXEN = 1;
		RCSTAbits.CREN = 1;
		return TRUE;
	}
	return FALSE;
}

BOOL xUARTClose( xUARTHandle xUART )
{
	if( xUART == 1 )
	{
		//RCSTA |= UART_SPEN_BIT; 	// Set SPEN bit
		TXSTAbits.TXEN = 0;
		RCSTAbits.CREN = 0;
		RCSTAbits.SPEN = 0;
		return TRUE;
	}
	return FALSE;
}

void vUARTWrite( xUARTHandle xUART, BYTE xTxChar )
{
	TXREG = xTxChar;		// Write to Tx register
	while( !(PIR1 & UART_TXIF_BIT) );		// Wait while transmission ends
}

void vUARTWriteWord( xUARTHandle, WORD xTxChar )
{
	if( TXSTA & UART_TX9_BIT )		// If 9 bit mode enabled
	{
		if( xTxChar & (1<<8) )		// Transmit ninth data bit
			TXSTA |= UART_TX9D_BIT;
		else
			TXSTA &= ~UART_TX9D_BIT;
	}	
	TXREG = (BYTE) xTxChar & 0x00FF;		// Write to Tx register
	while( !(PIR1 & UART_TXIF_BIT) );		// Wait while transmission ends
}

BYTE xUARTRead( xUARTHandle xUART )
{
	while( !(PIR1 & UART_RCIF_BIT) );	// Wait until data recieved
	return RCREG;
}

WORD xUARTReadWord( xUARTHandle xUART )
{
	WORD xRetData = 0;
	while( !(PIR1 & UART_RCIF_BIT) );	// Wait until data recieved
	if( RCSTA & UART_RX9_BIT )
	{
		if( RCSTA & UART_RX9D_BIT )
			xRetData |= (1<<8);
	}
	xRetData |= RCREG;
	return xRetData;
}

void vUARTWriteArray( xUARTHandle xUART, const char * pcTxBuffer, unsigned short usBufferLength )
{
	while( usBufferLength )
	{
		TXREG = *pcTxBuffer ++;			// Write to Tx register
		while( !(PIR1 & UART_TXIF_BIT) );	// Wait while transmission ends
		usBufferLength--;
	}
}

void vUARTReadArray( xUARTHandle xUART, char * pcRxBuffer, unsigned short usBufferLength )
{
	while ( usBufferLength )
	{
		while( !(PIR1 & UART_RCIF_BIT) );	// Wait until data recieved
		*pcRxBuffer ++ = RCREG;
		usBufferLength--;
	}
}

void vUARTPuts( xUARTHandle xUART, const char * pcString )
{
	while( *pcString != '\0' )
	{
		TXREG = * pcString ++;				// Write to Tx register
		while( !(PIR1 & UART_TXIF_BIT) );	// Wait while transmission ends
	}
}

void vUARTGets( xUARTHandle xUART, char * pcRxBuffer, unsigned short usBufferLength )
{
	while( *pcRxBuffer != '\0' && usBufferLength )
	{
		while( !(PIR1 & UART_RCIF_BIT) );	// Wait until data recieved
		*pcRxBuffer ++ = RCREG;
		usBufferLength--;
	}
}

BOOL xUARTAvailable( xUARTHandle xUART )
{
	return ( PIR1 & UART_RCIF_BIT ) ? TRUE : FALSE;
}
